--[[**********************************************************
    * Name:      wxiscan.lua
    * Purpose:   Demo for implementing Lua addons for wxIScan.
    * Author:    Daniel Nell (daniel.nell@nellresearch.de)
    * Created:   2009-11-13
    * Copyright: Daniel Nell (www.nellresearch.de)
    * License:   GPL
    **********************************************************
--]]


-------------------------------------------------------------
-- Redefine output function(s).
--
function print( ... )
    local args= {...}       -- Table for the function parameters.
    local text= ""          -- Accumulates the text.

    -- Iterate on the function parameters and concatenate
    for index, value in ipairs( args ) do
        text= text .. tostring( value ) .. "\t"
    end

    -- Output via LogMessage (wxIScan).
    LogMessage( text )
end


-------------------------------------------------------------
-- Functions calling menu handlers on the C++ side.
--
function ScanImage()
    return SendMenuEvent( ID_FILESCANIMAGE )
end

function PrintImage()
    return SendMenuEvent( ID_FILEPRINT )
end

function DocumentReadText( bAsync )
    if bAsync == nil then
        bAsync= true
    end
    if bAsync then
        return SendMenuEvent( ID_DOCUMENTREADTEXT )
    else
        return SendMenuEvent( ID_DOCUMENTREADTEXTSYNC )
    end
end

function DocumentPreviousPage()
    return SendMenuEvent( ID_DOCUMENTPREVIOUSPAGE )
end

function DocumentNextPage()
    return SendMenuEvent( ID_DOCUMENTNEXTPAGE )
end

function CmdStop()
    return SendMenuEvent( ID_CMDSTOP )
end


-------------------------------------------------------------
-- Main chunks (referrec by the config file)
--
function CarbonCopy()
    return ScanImage() and PrintImage()
end

function ScanAndRead()
    return ScanImage() and CmdStop() and DocumentReadText()
end

function TurnPagesAndRead()
    if CmdStop() and DocumentReadText( false ) then
        while DocumentNextPage() and CmdStop() and DocumentReadText() do
        end
    end
end

function Test()
    print( "LUA: Current date and time is " .. os.date( "%x %X", os.time() ) .. '.' )
    print( "LUA: Your home directory is '" .. os.getenv( "HOME" ) .. "'." )
end
