/***************************************************************
 * Name:      wxluascript.h
 * Purpose:   Defines a lua scripting object
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-11-11
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   wxWindows license
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

// wxWidgets headers
// ...

// Private header files.
#include "wxluascript.h"


// Scripting support.
//
// ...
//
wxLuaScript::wxLuaScript( wxEvtHandler *poParent, bool *pbParentState, const wxString& strScript )
{
    // Initialize members.
    m_poParent= poParent;
    m_strScript= strScript;
    m_pbParentState= pbParentState;
    if( m_pbParentState )
    {
        (*m_pbParentState)= true;
    }

    // Initialize Lua.
    L= lua_open();

    // Load Lua base libraries.
    luaL_openlibs( L );

    // Register "SendMenuEvent" as Lua function.
    RegisterFunction( wxT( "SendMenuEvent" ), SendMenuEvent );

    // Register "LogMessage" as Lua function.
    RegisterFunction( wxT( "LogMessage" ), LogMessage );

    // Register the this pointer (to the currenct wxLuaScript object)
    // as a global value called "this".
    lua_pushinteger( L, (long)this );
    lua_setglobal( L, "this" );
}

wxLuaScript::~wxLuaScript()
{
    // Cleanup Lua.
    lua_close( L );
}

wxLuaScript *wxLuaScript::GetThis( lua_State* L )
{
    // Get the this-pointer from Lua.
    lua_getglobal( L, "this" );
    return (wxLuaScript *)lua_tointeger( L, -1 );
}

int wxLuaScript::SendMenuEvent( lua_State* L )
{
    // Check the number of argument(s) and the type(s).
    if( ( lua_gettop( L ) != 1 ) || !lua_isnumber( L, 1 ) )
    {
        // Signal error.
        lua_pushstring( L, "Incorrect argument or incorrect number of arguments." );
        lua_error( L );
    }

    // Get the this pointer to the currenct wxLuaScript object.
    wxLuaScript *poThis= GetThis( L );

    // Process the corresponding wxWidgets menu event.
    //
    // NOTE: We get the adress of the this-pointer from Lua
    //       (for this is a static function, and we need it
    //       to call the correct event handler).
    wxCommandEvent oEvent( wxEVT_COMMAND_MENU_SELECTED, lua_tonumber( L, 1 ) );
    poThis->m_poParent->ProcessEvent( oEvent );

    // Push the return value ( the parent state );
    if( poThis->m_pbParentState )
    {
        lua_pushboolean( L, *poThis->m_pbParentState );
        (*poThis->m_pbParentState)= true;
    }
    else
    {
        lua_pushboolean( L, true );
    }

    // Return the number of (Lua) results.
	return 1;
}

int wxLuaScript::LogMessage( lua_State* L )
{
    // Check the number of argument(s) and the type(s).
    if( ( lua_gettop( L ) != 1 ) || !lua_isstring( L, 1 ) )
    {
        // Signal error.
        lua_pushstring( L, "Incorrect argument or incorrect number of arguments." );
        lua_error( L );
    }

    // Get the message and log it.
    ::wxLogMessage( wxString( (const char *)lua_tostring( L, 1 ), wxConvISO8859_1, wxSTRING_MAXLEN ) );

    // Return the number of (Lua) results.
	return 0;
}
