/***************************************************************
 * Name:      wxluascript.h
 * Purpose:   Defines a lua scripting object
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-11-11
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   wxWindows license
 **************************************************************/

#ifndef WXLUASCRIPT_H
#define WXLUASCRIPT_H

// Additional wxWidgets header file(s).
#include <wx/event.h>

// Include Lua header file(s).
#include "lua5.1/lua.hpp"

/////////////////////////////////////////////////////////////////////////////
//  Class wxLuaScript
//
/// \brief A class to handle Lua scripting.
///
/// ...
class wxLuaScript
{
  public:
    /// Standard constructor.
    wxLuaScript( wxEvtHandler *poParent, bool *pbParentState= NULL, const wxString& strScript= wxEmptyString );

    /// Virtual destructor.
    virtual ~wxLuaScript();

  public:
    /// Run the Lua script in memory.
    virtual void Run()
    {
        // Run the script.
        luaL_dostring( L, m_strScript.char_str() );
    }

    /// Register a new (LuaC-)function Lua.
    virtual void RegisterFunction( const wxString& strLuaCFunctionName, lua_CFunction pLuaCFunction )
    {
        lua_register( L, strLuaCFunctionName.char_str(), pLuaCFunction );
    }

    /// Register a string constant to Lua.
    virtual void RegisterConstant( const wxString& strLuaConstantName, const wxString& strValue )
    {
        lua_pushstring( L, strLuaConstantName.char_str() );
        lua_pushstring( L, strValue.char_str() );
        lua_settable( L, LUA_GLOBALSINDEX );
    }

    /// Register an integer constant to Lua.
    virtual void RegisterConstant( const wxString& strLuaConstantName, const int nValue )
    {
        lua_pushstring( L, strLuaConstantName.char_str() );
        lua_pushinteger( L, nValue );
        lua_settable( L, LUA_GLOBALSINDEX );
    }

    /// Call Lua function (procedure).
    virtual void CallLuaFunction( const wxString& strLuaFunction )
    {
        lua_getfield( L, LUA_GLOBALSINDEX, strLuaFunction.char_str() );
        lua_call( L, 0, 0 );
    }

  protected:
    // Static helper functions.
    static wxLuaScript *GetThis( lua_State* L );    ///< Get the this-pointer from Lua.
    // ...

  protected:
    // Functions accessible by the Lua language.
    static int SendMenuEvent( lua_State* L );       ///< Send a menu event to the parent event handler.
    static int LogMessage( lua_State* L );          ///< Log a message on the standard wxWidgets log target.
    // ...

  protected:
    wxEvtHandler *m_poParent;                       ///< Pointer to the parent event handler (most often the parent window).
    bool *m_pbParentState;                          ///< State of the parent object (true == OK, false == not OK).
    lua_State* L;                                   ///< Lua.
    wxString m_strScript;                           ///< Lua script as a string in memory.
};

#endif // WXLUASCRIPT_H
